package edu.rice.rubis.client;


import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.Naming;

import java.net.*;
import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;


// Danis Master Client for starting other clients over RMI

public class MainClient
{
	public static String targetMachines[] = { "192.168.0.30" ,  "192.168.0.80",  "192.168.0.45" /*  ,   "nukath09", "nukath10"
	 "192.168.1.50", "192.168.1.40", "192.168.1.30" */};

	public static String rubisURL = "http://192.168.0.10:8080/rubis/";
	// public static String rubisURL = "http://192.168.0.10:8080/rubis/";

	public static String statsURL = rubisURL + "stats.jsp";
	public static String webStatsURL = rubisURL + "webcachestats.jsp";

	public static String outputDir = "c:/temp/statistics";

	public static int port = 7000;

	public static int maxRuns = 25;

	public static int minNumClients = 33; // 400;

	public static int stepNumClients = 33; // 100;


    public static int maxCacheEntries = 2000;

    /*
	public static long rampUpMillis = 1000 * 1 * 5; // 60000;

    // Only used for experiment 1.
    public static long warmupMillis = 1000 * 1 * 5;

	public static long sessionMillis = 1000 * 1 * 5; // 300000;

	public static long rampDownMillis = 1000 * 1 * 4; // 60000;

    // Only used for experiment 1.
    public static int numWarmupClients = 50;
    */

	public static int rampUpMillis = 1000 * 60 * 1; // 60000;

    public static int warmupMillis = 1000 * 60 * 4;

	public static int sessionMillis = 1000 * 60 * 5; // 300000;

	public static int rampDownMillis = 1000 * 60 * 1; // 60000;

    public static int numWarmupClients = 150;


	public static void main(String[] args)
		throws Exception
	{
		if (args.length > 0) {
			outputDir = args[0];
		}

		try {
			System.setSecurityManager(new java.rmi.RMISecurityManager());
		}
		catch (java.rmi.RMISecurityException exc) {
			System.err.println("Security violation: " + exc.toString());
			System.exit(-1);
		}


		try {
            String baseFileName;
			IClientWrapper[] clients = new IClientWrapper[targetMachines.length];

			for (int i = 0; i < clients.length; i++) {
				clients[i] =
					(IClientWrapper) Naming.lookup("rmi://" + targetMachines[i] + ":" + port + "/rubis");
			}

            // Set base directory...
            DateFormat dateFormat = new SimpleDateFormat("MM-dd-yy-HHmm-ss");
            String dateString = dateFormat.format(new Date());
            callStatsPage("?dirname=c%3A%5Ctemp%5Cstatistics_" + dateString + "&setdir=Set");


            // Experiment 1:
            // Stepwise Increase Client Load
            // Variants:
            // - No caching
            // - JNDI caching only
            // - No replacement caching
            // - LRU caching
            // - LFU caching
            System.out.println("---- Experiment 1 ----");
            callStatsPage("?correctnessswitch=Submit");
            callStatsPage("?prefetchswitch=Submit");

			for (int j = 0; j < maxRuns; j++) {
					int numClients = minNumClients + j * stepNumClients;


					// be sure to disable Webcache
	            	callWebStatsPage("?webcacheswitch=Submit");
            		callWebStatsPage("?mediatorswitch=Submit");
            	    callWebStatsPage("?clearWebCache=clear+WebCache%2BMediator");

					System.out.println("Starting run with " + numClients + " clients.");

                    System.out.println("- No caching.");
                    callStatsPage("?clearcache=Clear+Cache");
					callStatsPage("?gc=Run+Garbage+Collection");
                    callStatsPage("?lookupswitch=Submit");
                    callStatsPage("?clearservletstats=Clear+Servlet+Stats");
                    callStatsPage("?clearcachestats=Clear+Cache+Stats");
					baseFileName = runClients(clients, numClients, sessionMillis,
                                              true, true, "nocache", true);
                    callStatsPage("?servletstats_info=" + numClients + "+clients&servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
                    callStatsPage("?cachestats_info=" + numClients + "+clients&cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");


                    System.out.println("- JNDI caching only.");
                    callStatsPage("?clearcache=Clear+Cache");
					callStatsPage("?gc=Run+Garbage+Collection");
                    callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
                    callStatsPage("?cacheswitch=Submit");
                    callStatsPage("?clearservletstats=Clear+Servlet+Stats");
                    callStatsPage("?clearcachestats=Clear+Cache+Stats");
					baseFileName = runClients(clients, numClients, sessionMillis,
                                              true, true, "jndionly", true);
                    callStatsPage("?servletstats_info=" + numClients + "+clients&servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
                    callStatsPage("?cachestats_info=" + numClients + "+clients&cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");


                    System.out.println("- No replacement caching.");
                    callStatsPage("?clearcache=Clear+Cache");
					callStatsPage("?gc=Run+Garbage+Collection");
                    callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
                    callStatsPage("?cacheswitch_value=true&cacheswitch=Submit");
                    callStatsPage("?strategy=none&replacement=Submit");
                    // Warm up cache...
                    // TODO: shorten warmup time!
					runClients(clients, numWarmupClients, warmupMillis,
                               true, false, "warmup", true);
                    callStatsPage("?clearservletstats=Clear+Servlet+Stats");
                    callStatsPage("?clearcachestats=Clear+Cache+Stats");
					baseFileName = runClients(clients, numClients, sessionMillis,
                                              true, true, "norepl", true);
                    callStatsPage("?servletstats_info=" + numClients + "+clients&servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
                    callStatsPage("?cachestats_info=" + numClients + "+clients&cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");


                    System.out.println("- LRU caching.");
                    callStatsPage("?clearcache=Clear+Cache");
					callStatsPage("?gc=Run+Garbage+Collection");
                    callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
                    callStatsPage("?cacheswitch_value=true&cacheswitch=Submit");
                    callStatsPage("?strategy=lru&replacement=Submit");
                    callStatsPage("?maxentries_value=" + maxCacheEntries + "&maxentries=Submit");
                    // Warm up cache...
					runClients(clients, numWarmupClients, warmupMillis,
                               true, false, "warmup", true);
                    callStatsPage("?clearservletstats=Clear+Servlet+Stats");
                    callStatsPage("?clearcachestats=Clear+Cache+Stats");
					baseFileName = runClients(clients, numClients,  sessionMillis,
                                              true, true, "lru", true);
                    callStatsPage("?servletstats_info=" + numClients + "+clients&servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
                    callStatsPage("?cachestats_info=" + numClients + "+clients&cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");


                    System.out.println("- LFU caching.");
                    callStatsPage("?clearcache=Clear+Cache");
					callStatsPage("?gc=Run+Garbage+Collection");
                    callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
                    callStatsPage("?cacheswitch_value=true&cacheswitch=Submit");
                    callStatsPage("?strategy=lfu&replacement=Submit");
                    callStatsPage("?maxentries_value=" + maxCacheEntries + "&maxentries=Submit");
                    // Warm up cache...
					runClients(clients, numWarmupClients, warmupMillis,
                               true, false, "warmup", true);
                    callStatsPage("?clearservletstats=Clear+Servlet+Stats");
                    callStatsPage("?clearcachestats=Clear+Cache+Stats");
					baseFileName = runClients(clients, numClients, sessionMillis,
                                              true, true, "lfu", true);
                    callStatsPage("?servletstats_info=" + numClients + "+clients&servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
                    callStatsPage("?cachestats_info=" + numClients + "+clients&cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");
/*
                    // ----------------------  webcache noreplacement ----------------------------


		    System.out.println("-webcache noreplacement");

                    callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
                    callStatsPage("?cacheswitch_value=true&cacheswitch=Submit");
                    callStatsPage("?strategy=none&replacement=Submit");


					// enable the webcache
            	    callWebStatsPage("?webcacheswitch_value=true&webcacheswitch=Submit");
            	    callWebStatsPage("?mediatorswitch_value=true&mediatorswitch=Submit");
            		callWebStatsPage("?setCacheFolder=c%3A%5Cwebcachedata");
            	    callWebStatsPage("?clearWebCache=clear+WebCache%2BMediator");
					// warm up

					runClients(clients, numWarmupClients, warmupMillis,
                               true, false, "warmup", true);
                    callStatsPage("?clearservletstats=Clear+Servlet+Stats");
                    callStatsPage("?clearcachestats=Clear+Cache+Stats");
 		    baseFileName = runClients(clients, numClients, sessionMillis,
                                              true, true, "norepl_webcache", true);
                    callStatsPage("?servletstats_info=" + numClients + "+clients&servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
                    callStatsPage("?cachestats_info=" + numClients + "+clients&cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");

                    // ----------------------  webcache lru -------------------------------------

		    System.out.println("-webcache lru");


					// enable the webcache
	            	callWebStatsPage("?webcacheswitch_value=true&webcacheswitch=Submit");
            		callWebStatsPage("?mediatorswitch_value=true&mediatorswitch=Submit");
            		callWebStatsPage("?setCacheFolder=c%3A%5Cwebcachedata");
            		callWebStatsPage("?clearWebCache=clear+WebCache%2BMediator");

	    		// lru
	                callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
                        callStatsPage("?cacheswitch_value=true&cacheswitch=Submit");
                        callStatsPage("?strategy=lru&replacement=Submit");
                        callStatsPage("?maxentries_value=" + maxCacheEntries + "&maxentries=Submit");



					// warm up
					runClients(clients, numWarmupClients, warmupMillis,
                               true, false, "warmup", true);
                    callStatsPage("?clearservletstats=Clear+Servlet+Stats");
                    callStatsPage("?clearcachestats=Clear+Cache+Stats");
		    baseFileName = runClients(clients, numClients, sessionMillis,
                                              true, true, "lru_webcache", true);
                    callStatsPage("?servletstats_info=" + numClients + "+clients&servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
                    callStatsPage("?cachestats_info=" + numClients + "+clients&cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");

                  // ------------------------ disable webache --------------------------------

	            	callWebStatsPage("?webcacheswitch=Submit");
            		callWebStatsPage("?mediatorswitch=Submit");
            		*/
			}



            // Experiment 2:
            // Hit Miss ration over time...
            // Variants:
            // - LRU
            // - No replacement
            System.out.println("---- Experiment 2 ----");
            callStatsPage("?correctnessswitch=Submit");
            callStatsPage("?prefetchswitch=Submit");

            rampUpMillis = 1;
            sessionMillis = 1000; // 1000 * 60 * 120;
            rampDownMillis = 1;

            System.out.println("- LRU hit/miss ratio ");
            callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
            callStatsPage("?cacheswitch_value=true&cacheswitch=Submit");
            callStatsPage("?strategy=lru&replacement=Submit");
            callStatsPage("?maxentries_value=" + maxCacheEntries + "&maxentries=Submit");
            callStatsPage("?hitmiss_calls=50&hitmiss_on=Start+Logging");
            callStatsPage("?clearservletstats=Clear+Servlet+Stats");
            callStatsPage("?clearcachestats=Clear+Cache+Stats");

			baseFileName = runClients(clients, 600, sessionMillis,
                                      true, true, "lru_hitmiss", false);
            callStatsPage("?servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
            callStatsPage("?cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");
            callStatsPage("?hitmiss_filename=hitmissstats" + baseFileName + ".ser&hitmiss_off=Save+File+and+Stop+Logging");

            System.out.println("- No replacement hit/miss ratio ");
            callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
            callStatsPage("?cacheswitch_value=true&cacheswitch=Submit");
            callStatsPage("?strategy=none&replacement=Submit");
            callStatsPage("?hitmiss_calls=50&hitmiss_on=Start+Logging");
            callStatsPage("?clearservletstats=Clear+Servlet+Stats");
            callStatsPage("?clearcachestats=Clear+Cache+Stats");
			baseFileName = runClients(clients, 600,  sessionMillis,
                                      true, true, "norepl_hitmiss", false);
            callStatsPage("?servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
            callStatsPage("?cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");
            callStatsPage("?hitmiss_filename=hitmissstats" + baseFileName + ".ser&hitmiss_off=Save+File+and+Stop+Logging");




            // Experiment 3:
            // Replacement strategy stress test...
            // Variants:
            // - LRU
            // - LFU
            // - No replacement
            System.out.println("---- Experiment 3 ----");
            callStatsPage("?correctnessswitch=Submit");
            callStatsPage("?prefetchswitch=Submit");

            rampUpMillis = 1;
            sessionMillis = 1000; // 1000 * 60 * 120;
            rampDownMillis = 1;

            System.out.println("- LRU memory stress test ");
            callStatsPage("?strategy=lru&replacement=Submit");
            callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
            callStatsPage("?cacheswitch_value=true&cacheswitch=Submit");
            callStatsPage("?maxentries_value=" + maxCacheEntries + "&maxentries=Submit");
            callStatsPage("?clearcache=Clear+Cache");
            callStatsPage("?withgc=true&calls=50&memlog_filename=lru_memlog.ser&memlog_on=Start+Logging");
			baseFileName = runClients(clients, 600, sessionMillis,
                                      true, true, "lru_memstress", false);
            callStatsPage("?memlog_off=Close+File+and+Stop");
            callStatsPage("?servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
            callStatsPage("?cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");

            System.out.println("- LFU memory stress test ");
            callStatsPage("?strategy=lfu&replacement=Submit");
            callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
            callStatsPage("?cacheswitch_value=true&cacheswitch=Submit");
            callStatsPage("?maxentries_value=" + maxCacheEntries + "&maxentries=Submit");
            callStatsPage("?clearcache=Clear+Cache");
            callStatsPage("?withgc=true&calls=50&memlog_filename=lfu_memlog.ser&memlog_on=Start+Logging");
			baseFileName = runClients(clients, 600, sessionMillis,
                                      true, true, "lfu_memstress", false);
            callStatsPage("?memlog_off=Close+File+and+Stop");
            callStatsPage("?servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
            callStatsPage("?cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");

            System.out.println("- No replacement memory stress test ");
            callStatsPage("?strategy=none&replacement=Submit");
            callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
            callStatsPage("?cacheswitch_value=true&cacheswitch=Submit");
            callStatsPage("?clearcache=Clear+Cache");
            callStatsPage("?withgc=true&calls=50&memlog_filename=norepl_memlog.ser&memlog_on=Start+Logging");
			baseFileName = runClients(clients, 600,  sessionMillis,
                                      true, true, "norepl_memstress", false);
            callStatsPage("?memlog_off=Close+File+and+Stop");
            callStatsPage("?servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
            callStatsPage("?cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");


            // Experiment 4:
            // Correctness test (with LRU replacement)
            System.out.println("---- Experiment 4 ----");

            rampUpMillis = 1;
            sessionMillis = 1000; // 1000 * 60 * 120;
            rampDownMillis = 1;
            System.out.println("- No replacement correctness stress test ");
            callStatsPage("?strategy=none&replacement=Submit");
            callStatsPage("?lookupswitch_value=true&lookupswitch=Submit");
            callStatsPage("?cacheswitch_value=true&cacheswitch=Submit");
            callStatsPage("?correctnessswitch_value=true&correctnessswitch=Submit");
            callStatsPage("?clearcache=Clear+Cache");
            callStatsPage("?correctness_clear=Clear+Correctness+Info");
			baseFileName = runClients(clients, 600, sessionMillis,
                                      true, true, "norepl_correctness", true);
            callStatsPage("?servletstats_filename=servletstats" + baseFileName + ".ser&saveservletstats=Save");
            callStatsPage("?cachestats_filename=cachstats" + baseFileName + ".ser&savecachestats=Save");


		}
		catch (RemoteException e) {
			System.err.println("Remote error: " + e.toString());
		}
	}


	private static void callStatsPage(String urlParam)
        throws Exception
	{
        callPage(statsURL, urlParam);
    }



	private static void callWebStatsPage(String urlParam)
        throws Exception
	{
        callPage(webStatsURL, urlParam);
    }


    private static void callPage(String baseUrl, String urlParam)
        throws Exception
    {
        Exception finalException = null;

        for (int i = 1; i <= 5; i++) {
            try {
		        URL statsPage = new URL(baseUrl + urlParam);
		        URLConnection connection = statsPage.openConnection();

		        connection.getInputStream().close();

                return;

		    }  catch ( Exception e ) {
                System.out.println("Try " + i + ": Calling '" + baseUrl + urlParam + "' failed.");
                finalException = e;
		    }

            Thread.currentThread().sleep(1000 * 60);
        }

        System.out.println("Calling '" + statsURL + urlParam + "' failed.");
        throw finalException;
	}


	public static String runClients(IClientWrapper[] clients,
                                    int numClients,
                                    long sessionMillis,
									boolean thinkTime, boolean biddingMix,
                                    String extension,
                                    boolean attach)
		throws Exception
	{
        String mainFileName = getMainFileName(numClients,
                                              sessionMillis,
								              thinkTime, biddingMix,
                                              extension,
                                              attach);

		for (int i = 0; i < clients.length; i++) {
            clients[i].startClient(i,
                                   numClients,
                                   rampUpMillis,
                                   sessionMillis,
                                   rampDownMillis,
								   thinkTime, biddingMix,
                                   "id" + i + "_" + mainFileName);
		}

		Thread.currentThread().sleep(rampUpMillis + sessionMillis + rampDownMillis + 30000);

		// Poll until done...
		boolean allDone = false;
		while (!allDone) {
			allDone = true;
			for (int i = 0; i < clients.length; i++) {
				allDone = allDone && clients[i].isDone();
			}

			Thread.currentThread().sleep(10000);
		}

        return mainFileName;
	}


    public static String getMainFileName(int numClients,
                                  long sessionMillis,
								  boolean thinkTime,
                                  boolean biddingMix,
                                  String extension,
                                  boolean attach)
    {
        return (attach ? "" : "_numc" + numClients) +
			   "_sess" + sessionMillis +
			   "_bidmix" + biddingMix +
			   "_thinkTime" + thinkTime +
			   "_" + extension;
    }
}






